/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.stat.handler;

import java.io.File;
import java.io.IOException;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.xml.sax.SAXException;

import benten.cat.glossary.core.BentenGlossaryEngine;
import benten.cat.stat.CatStatPlugin;
import benten.cat.stat.model.GlossaryHitRateModel;
import benten.core.model.Xliff;
import blanco.xliff.valueobject.BlancoXliffTransUnit;

/**
 * 用語対訳集ヒット率取得ハンドラーです。
 */
public class ShowGlossaryHitRateHandler extends AbstractStatHandler<GlossaryHitRateModel> {

	private static final Set<String> cache = new HashSet<String>();

	@Override
	protected GlossaryHitRateModel processFile(File file) throws IOException, SAXException {
		GlossaryHitRateModel model = new GlossaryHitRateModel();
		model.setFile(file);
		Xliff xliff = Xliff.loadInstance(file);
		List<BlancoXliffTransUnit> unitList = xliff.getAllTransUnitList();
		for (BlancoXliffTransUnit unit : unitList) {
			String source = unit.getSource();
			if (source != null) {
				setHitRate(source, model);
			}
		}
		return model;
	}

	protected void setHitRate(String source, GlossaryHitRateModel model) {
		BentenGlossaryEngine glossaryEngine = CatStatPlugin.getGlossaryEngine();
		String[] words = splitWord(source);
		int hit = 0;
		for (String word : words) {
			if (cache.contains(word)) {
				hit++;
			} else {
				if (glossaryEngine.search(word).size() > 0) {
					hit++;
					cache.add(word);
				}
			}
		}
		int rate = 100 * hit / words.length;
		if (rate >= 75) {
			model.setHit_75(model.getHit_75() + 1);
		} else if (rate >= 50) {
			model.setHit_50(model.getHit_50() + 1);
		} else if (rate >= 25) {
			model.setHit_25(model.getHit_25() + 1);
		} else {
			model.setHitUnder_25(model.getHitUnder_25() + 1);
		}
	}
}
