/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.stat.handler;

import java.io.File;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import org.xml.sax.SAXException;

import benten.cat.stat.model.TranslationRateModel;
import benten.core.model.Xliff;
import blanco.xliff.valueobject.BlancoXliffTarget;
import blanco.xliff.valueobject.BlancoXliffTransUnit;

/**
 * 翻訳率取得ハンドラーです。
 */
@SuppressWarnings("serial")
public class ShowTranslationRateHandler extends AbstractStatHandler<TranslationRateModel> {

	/** 状態 - 未翻訳 */
	private static final List<String> untranslated = new LinkedList<String>() {
		{
			add(null);
			add("");
			add("new");
			add("needs-translation");
		}
	};

	/** 状態 - 翻訳済 */
	private static final List<String> translated = new LinkedList<String>() {
		{
			add("x-reviewed");
			add("final");
			add("signed-off");
		}
	};

	@Override
	protected TranslationRateModel processFile(File file) throws IOException, SAXException {
		Xliff xliff = Xliff.loadInstance(file);
		List<BlancoXliffTransUnit> unitList = xliff.getAllTransUnitList();

		TranslationRateModel model = new TranslationRateModel();
		model.setFile(file);
		model.setTransUnit(unitList.size());

		for (BlancoXliffTransUnit unit : unitList) {
			BlancoXliffTarget target = unit.getTarget();
			String state = target != null ? target.getState() : null;
			if (untranslated.contains(state)) {
				model.setUntranslated(model.getUntranslated() + 1);
			} else if (translated.contains(state)) {
				model.setTranslated(model.getTranslated() + 1);
			} else {
				model.setWorking(model.getWorking() + 1);
			}
		}

		model.setUntranslatedRate(getRate(model.getUntranslated(), unitList.size()));
		model.setWorkingRate(getRate(model.getWorking(), unitList.size()));
		model.setTranslatedRate(getRate(model.getTranslated(), unitList.size()));
		return model;
	}

	private int getRate(int size, int totalSize) {
		return (int) Math.ceil(100d * size / totalSize);
	}
}
